package test;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;

import junit.framework.TestCase;

import org.mozilla.javascript.Context;
import org.mozilla.javascript.ScriptableObject;

public class OptimizationBug extends TestCase {

    private Context cx;
    private ScriptableObject scope;

    protected void setUp() throws Exception {
        super.setUp();

    }

    private void initContextAndScope(int optimizationLevel) {
        cx = Context.enter();
        cx.setOptimizationLevel(optimizationLevel);

        scope = cx.initStandardObjects();
    }

    protected void tearDown() throws Exception {
        Context.exit();
        super.tearDown();
    }

    public void testBugWithNoOptimization() {
        initContextAndScope(0);
        runScript("testBug.js");
    }

    // Test fails!
    public void testBugWithOptimization() {
        initContextAndScope(9);
        runScript("testBug.js");
    }

    public void testWorkaroundWithNoOptimization() {
        initContextAndScope(0);
        runScript("testWorkaround.js");
    }

    public void testWorkaroundWithOptimization() {
        initContextAndScope(9);
        runScript("testWorkaround.js");
    }

    private void runScript(String scriptName) {
        String script = getScriptSource(scriptName);
        cx.evaluateString(scope, script, scriptName, 1, null);
    }

    private String getScriptSource(String scriptName) {
        final InputStream scriptStream = getClass().getResourceAsStream(scriptName);
        if (scriptStream == null) {
            throw new RuntimeException("Cannot find resource " + scriptName);
        }
        InputStream stream = new BufferedInputStream(scriptStream);
        StringBuffer sb = new StringBuffer();
        int c;
        try {
            while ((c = stream.read()) != -1) {
                sb.append((char) c);
            }
        }
        catch (IOException e) {
            e.printStackTrace();
        }
        return sb.toString();
    }

}
